import io

import pandas as pd

from datetime import datetime
from typing import List, Optional

from pathlib import Path
from exif2pandas import extract
import pkg_resources


class Where:
    def __init__(
            self,
            pictures_root: List[Path],
            feather_location = None,
            cities_file=None,
            ignore_countries=(),
            processes=5,
            first_interval: Optional[datetime.date] = None,
            last_interval: Optional[datetime.date] = None,
    ):
        self.cities_file = cities_file
        if cities_file is None:
            self.cities_file = pkg_resources.resource_filename('photos_where', 'cities.csv')

        self.ignore_countries = ignore_countries
        self.feather_location = feather_location
        self.first_interval = first_interval
        self.last_interval = last_interval

        # get raw dataframe with wide exif rows
        self.raw_exif_df = extract.extract_feather(
            feather_path=Path(feather_location).resolve(),
            pictures_root=pictures_root,
            processes=processes
        )

        # extract country name from gps data
        self.countries_df = self._get_countries_df()

        # create intervals based on the dates and gps
        self.intervals = self._get_intervals()
        self.intervals_df = pd.DataFrame(self.intervals)

        # create dataframe where every day has country code:
        self.day_df = self._get_country_per_day_df()


    def _get_countries_df(self):
        """
        Returns dataframe with country and city name taken from gps data
        """
        import reverse_geocoder
        if self.cities_file:
            with open(self.cities_file, encoding='utf-8') as f:
                geocoder = reverse_geocoder.RGeocoder(
                    mode=2,
                    verbose=True,
                    stream=io.StringIO(f.read())
                )
        else:
            print('using geocoder default cities file')
            geocoder = reverse_geocoder.RGeocoder(
                mode=2,
                verbose=True,
                stream=None,
            )

        # for the location purposed we can drop all the other columns,
        # this is needed for .dropna
        df_location = (
            self.raw_exif_df[['cleaned_longitude', 'cleaned_latitude', 'cleaned_date']]
            .dropna()
            .set_index('cleaned_date')
        )

        tuple_locs = [
            (float(x), float(y))
            for x, y in
            df_location[
                ['cleaned_latitude', 'cleaned_longitude']
            ].itertuples(index=False)
        ]
        country_df = pd.DataFrame(geocoder.query(tuple_locs))
        # add date index, geocoder preservers the
        # order of the rows so this is fine
        country_df.index = df_location.index
        df = df_location.join(country_df)
        df.sort_index(inplace=True)
        return df


    def _get_intervals(self):
        first_row = self.countries_df[:1].to_dict(orient='index')
        previous_date = list(first_row.keys())[0]
        previous_country_code = first_row[previous_date]['cc']
        previous_country = first_row[previous_date]['admin1']


        if self.first_interval:
            previous_date = self.first_interval

        intervals = []

        for dt, row in self.countries_df.iterrows():
            country_code = row['cc']
            country = row['admin1']

            if country_code in self.ignore_countries:
                continue

            if country_code != previous_country_code:
                days = (dt - previous_date).days

                # if days == 0:
                #     continue

                intervals.append({
                    'from': previous_date.date(),
                    'to': dt.date().isoformat(),
                    'days': days,
                    'country': previous_country,
                    'country_code': previous_country_code
                })
                previous_date = dt
                previous_country_code = country_code
                previous_country = country


        # add the last interval
        if self.last_interval:
            last_interval = self.last_interval

        else:
            last_interval = datetime.today()

        days = (last_interval - previous_date).days
        intervals.append({
            'from': previous_date.date(),
            'to': last_interval.date().isoformat(),
            'days': days,
            'country': previous_country,
            'country_code': previous_country_code
        })

        return intervals


    def _get_country_per_day_df(self):
        intervals_full = []

        for i in self.intervals:
            for day in pd.date_range(i['from'], i['to']):
                intervals_full.append({
                    'day': day,
                    'country': i['country'],
                    'country_code': i['country_code']
                })

        df_days = pd.DataFrame(intervals_full).set_index('day').astype({
            'country': 'string',
            'country_code': 'string',
        })
        return df_days


#!/usr/bin/env python
# -*- coding: utf-8 -*-


try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup

with open("README.md", "r") as fh:
    long_description = fh.read()

setup(
    name="photos_where",
    version="1.3",
    description="Analyze exif data",
    long_description=long_description,
    long_description_content_type="text/markdown",
    author="Visgean",
    author_email="visgean@gmail.com",
    url="https://github.com/visgean/where",
    packages=[
        "photos_where",
    ],
    package_dir={"photos_where": "photos_where"},
    license="MIT",
    keywords="exif sql",
    classifiers=[
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.7",
    ],
    install_requires=[
        "exif2pandas",
        "pandas",
        "matplotlib",
        "reverse-geocoder==1.5.1",
        "pyarrow",
    ],
    entry_points={"console_scripts": ["photos_where = photos_where.main:main"]},
)



#!/usr/bin/env python3
import argparse
import os

from pathlib import Path
import matplotlib.pyplot as plt

from matplotlib.pyplot import savefig

from . import Where

parser = argparse.ArgumentParser(description="Generate analysis from your photos")
parser.add_argument("picture_folders", nargs="+", help="list of folders with the images")

parser.add_argument(
    "-f",
    "--feather",
    default='where/photos.feather',
    help="Output the data frame to feather file (this will override existing file!)",
)

parser.add_argument(
    '-o',
    '--output'
    "out", help="Output folder for the csv etc", default='where',
    dest='out'
)

parser.add_argument(
    '-i'
    "--ignore_countries",
    nargs="+",
    help="Country codes to ignore",
    dest='ignore'

)

parser.add_argument(
    "-p",
    "--processes",
    type=int,
    help="number of processes to use for collecting exif data, defaults to 5",
    default=5,
)


def main():
    args = parser.parse_args()

    out_dir = Path(args.out)
    os.makedirs(out_dir, exist_ok=True)
    picture_dirs = [Path(f).resolve() for f in args.picture_folders]

    feather_loc = None
    if args.feather:
        feather_loc = Path(args.feather).resolve()

    ignore_countries = []
    if args.ignore:
        ignore_countries = args.ignore

    where = Where(
        feather_location=feather_loc,
        pictures_root=picture_dirs,
        processes=5,
        ignore_countries=ignore_countries
    )

    where.day_df.to_csv(out_dir / 'location-by-day.csv')
    where.intervals_df.to_csv(out_dir / 'intervals.csv')

    where.countries_df.cc.value_counts().plot(kind='pie')
    savefig(out_dir / 'countries-pie.jpg')

    where.countries_df.admin2.value_counts().plot(kind='pie')
    savefig(out_dir / 'cities-pie.jpg')

    plt.style.use('seaborn-deep')
    fig = plt.figure(figsize=(15, 20))
    ax = fig.gca()

    where.day_df['country_code'].hist(by=where.day_df.index.year, ax=ax, rwidth=0.5)
    savefig(out_dir / 'years.jpg')


if __name__ == "__main__":
    main()
